﻿/*
 * Created by SharpDevelop.
 * User: Anthony
 * Date: 5/27/2017
 * Time: 7:38 PM
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using System.Security.Permissions;

using THPS;

namespace FntViewer
{
	/// <summary>
	/// Description of MainForm.
	/// </summary>
	public partial class MainForm : Form
	{
		THPSVersionConfig [] versions;
		int version = 0;
		Fnt font;
		bool SetHex = false;
		int selected = 0;
		
		public MainForm()
		{
			InitializeComponent();
			
			versions = new THPSVersionConfig[]
			{
				new THPSVersionConfig(THPS_VERSION.THPS4, "Tony Hawk's Pro Skater 4", "C:\\Program Files (x86)\\Aspyr\\Tony Hawks Pro Skater 4\\Game\\data\\fonts"),
				new THPSVersionConfig(THPS_VERSION.THUG, "Tony Hawk's Underground", "C:\\Program Files (x86)\\Activision\\Tony Hawk's Underground\\Game\\Data\\fonts"),
				new THPSVersionConfig(THPS_VERSION.THUG2, "Tony Hawk's Underground 2", "C:\\Program Files (x86)\\Activision\\Tony Hawk's Underground 2\\Game\\Data\\fonts")
			};
			
			font = new Fnt();
		}
		
		void UpdateCharList()
		{
			selected = listBoxChars.SelectedIndex;
			
			listBoxChars.Items.Clear();
					
			for(int i = 0; i < font.NumChars; i++)
			{
				listBoxChars.Items.Add(Convert.ToChar(font.Chars[i].Ascii));
			}
			
			if(listBoxChars.Items.Count <= selected)
			{
				selected = 0;
			}
			
			listBoxChars.SelectedIndex = selected;
		}
		
		void ButtonOpenClick(object sender, EventArgs e)
		{
			openFileDialog1.InitialDirectory = Path.GetFullPath(textBoxFilename.Text);
			DialogResult dr = openFileDialog1.ShowDialog();
			
			if(dr == DialogResult.OK)
			{				
				selected = 0;
				bool success = false;
				
				switch((THPS_VERSION)version)
				{
					case THPS_VERSION.THPS4:
						success = font.OpenTHUGFile(openFileDialog1.FileName);
						break;
					case THPS_VERSION.THUG:
						success = font.OpenTHUGFile(openFileDialog1.FileName);
						break;
					case THPS_VERSION.THUG2:
						success = font.OpenTHUG2File(openFileDialog1.FileName);
						break;
				}
				
				if(success)
				{
					textBoxFilename.Text = openFileDialog1.FileName;
					versions[version].path = Path.GetFullPath(openFileDialog1.FileName);
					UpdateCharList();
				}
				else
				{
					MessageBox.Show("Could not open file " + openFileDialog1.FileName +"\n\nTry running program as Administrator", "Error",
					MessageBoxButtons.OK);
				}
			}
			else
			{
				return;
			}
		}
		
		void ButtonSaveClick(object sender, EventArgs e)
		{
			saveFileDialog1.InitialDirectory = Path.GetFullPath(textBoxFilename.Text);
			saveFileDialog1.FileName = Path.GetFileName(textBoxFilename.Text);
			
			FileStream _fs;
			
			saveFileDialog1.ShowDialog();
			
			if(saveFileDialog1.FileName != "")
			{
				try
				{
					_fs = (FileStream)saveFileDialog1.OpenFile();
				}
				catch
				{
					return;
				}
				
				switch((THPS_VERSION)version)
				{
					case THPS_VERSION.THPS4:
						font.SaveTHUGFile(_fs);
						break;
					case THPS_VERSION.THUG:
						font.SaveTHUGFile(_fs);
						break;
					case THPS_VERSION.THUG2:
						font.SaveTHUG2File(_fs);
						break;
				}
			}
			else
			{
				return;
			}
		}
		
		void MainFormLoad(object sender, EventArgs e)
		{			
			for(int i = 0; i < versions.Length; i++)
			{
				comboBoxVersion.Items.Add(versions[i].text);
				if(Directory.Exists(versions[i].path))
				{
					version = i;
				}
			}
			
			comboBoxVersion.SelectedIndex = version;
		}
		
		void ComboBoxVersionSelectedIndexChanged(object sender, EventArgs e)
		{
			version = comboBoxVersion.SelectedIndex;
			textBoxFilename.Text = versions[version].path;
			openFileDialog1.InitialDirectory = versions[version].path;
			saveFileDialog1.InitialDirectory = versions[version].path;
			
			if(Directory.Exists(versions[version].path))
			{
				labelFound.ForeColor = Color.Green;
			   	labelFound.Text = "Found";
			}
			else
			{
				labelFound.ForeColor = Color.Red;
				labelFound.Text = "Not Found";
			}
			
			textBoxFilename.Select();
		}
		
		void ListBoxCharsSelectedIndexChanged(object sender, EventArgs e)
		{
			selected = listBoxChars.SelectedIndex;
			
			textBoxBaseline.Text = Convert.ToString(font.Chars[selected].Baseline);
			textBoxAscii.Text = Convert.ToChar(font.Chars[selected].Ascii).ToString();
			textBoxHex.Text = String.Format("{0:X}", font.Chars[selected].Ascii);
			
			panel1.Refresh();
		}
		
		void ButtonSetBaselineClick(object sender, EventArgs e)
		{
			int i = listBoxChars.SelectedIndex;
			
			font.Chars[i].Baseline = Convert.ToInt16(textBoxBaseline.Text);
			
			UpdateCharList();
		}
		
		void TextBoxAsciiTextChanged(object sender, EventArgs e)
		{
			SetHex = false;
		}
		
		void TextBoxHexTextChanged(object sender, EventArgs e)
		{
			SetHex = true;
		}
		
		void Button1Click(object sender, EventArgs e)
		{
			int i = listBoxChars.SelectedIndex;
			
			if(SetHex == true)
			{
				font.Chars[i].Ascii = (ushort)Byte.Parse(textBoxHex.Text, System.Globalization.NumberStyles.HexNumber);
			}
			else
			{
				font.Chars[i].Ascii = (ushort)textBoxAscii.Text[0];
			}
			
			UpdateCharList();
		}
		
		void Panel1Paint(object sender, PaintEventArgs e)
		{
			bool black = true;
			
			for(int y = 0; y < 128; y += 5)
			{
				for(int x = 0; x < 128; x += 5)
				{
					switch(black)
					{
						case true:
							e.Graphics.FillRectangle(Brushes.LightGray, x, y, 5, 5);
							break;
						case false:
							e.Graphics.FillRectangle(Brushes.White, x, y, 5, 5);
							break;
					}
					
					black = !black;
				}
				black = !black;
			}
			
			if(font.NumChars > 0)
			{
				int scale = 1;
				
				switch(font.Chars[selected].Width > font.Chars[selected].Height)
				{
					case true:
						scale = 128 / font.Chars[selected].Width;
						break;
					case false:
						scale = 128 / font.Chars[selected].Height;
						break;
				}
				
				e.Graphics.DrawImage(font.Chars[selected].BMP, 0, 0, font.Chars[selected].Width * scale, font.Chars[selected].Height * scale);
				e.Graphics.DrawLine(Pens.Red, 0, font.Chars[selected].Baseline * scale, 128, font.Chars[selected].Baseline * scale);
			}
			
		}
	}
}
