﻿/*
 * Created by SharpDevelop.
 * User: Anthony
 * Date: 6/11/2017
 * Time: 8:24 AM
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */

using System;
using System.Collections.Generic;
using System.IO;
using System.Drawing;

namespace THPS
{	
	public enum THPS_VERSION
	{
		THPS4 = 0,
		THUG,
		THUG2
	}
	
	public class THPSVersionConfig
	{
		public THPS_VERSION		version;
		public string 			text;
		public string 			path;
		
		public THPSVersionConfig(THPS_VERSION a, string b, string c)
		{
			version = a;
			text = b;
			path = c;
		}
	}
	
	public struct RGBA
	{
		public byte	R;
		public byte G;
		public byte B;
		public byte A;
	}
	
	public class FntChar
	{
		public short 	Baseline;
		public ushort	Ascii;
		public ushort	PosX;
		public ushort	PosY;
		public ushort	Width;
		public ushort	Height;
		public Bitmap	BMP;
		
		public FntChar(short a, ushort b)
		{
			Baseline	= a;
			Ascii		= b;
		}
	}
	
	public struct TextureData
	{
		public Int32	DataLength;
		public Int16	Width;
		public Int16	Height;
		public Int16	Depth;
		public byte[]	Extra;
	}
	
	public class Fnt
	{
		public string			Filename;
		public int				FileLength;
		public FileStream		File;
		public bool				FileOpen;
		public byte[]			FileBuffer;
		public int				NumChars;
		public List<FntChar>	Chars;
		public Int32			Height;
		public Int32			HeightOverBase;
		public TextureData		Texture;
		public byte[]			Pixels;
		public uint				PixelDataSize;
		
		public RGBA[]			CLUT;
		
		int bitbytes(int bits)
		{
			int bytes = bits + 7;
			bytes &= ~7;
			bytes >>= 3;
			
			return bytes;
		}
		
		public Fnt()
		{
			FileOpen = false;
			NumChars = 0;
			Chars = new List<FntChar>();
			
			Texture.Extra = new byte[6];
			CLUT = new RGBA[256];
		}
		
		~Fnt()
		{
			CloseFile();
		}
		
		public void CloseFile()
		{
			if(FileOpen)
			{
				File.Close();
				FileOpen = false;
				File = null;
			}
			
			if(NumChars > 0)
			{
				Chars.Clear();
				NumChars = 0;
			}
		}
		
		public bool OpenTHUGFile(string a)
		{
			CloseFile();
			
			Filename = a;
			
			try
			{
				File = new FileStream(Filename, FileMode.Open);
			}
			catch
			{
				return false;
			}
			
			if(File.CanRead)
			{
				FileOpen = true;
				FileBuffer = new byte[File.Length];
				File.Read(FileBuffer, 0, FileBuffer.Length);
				
				int fileptr = 0;
				
				FileLength = BitConverter.ToInt32(FileBuffer, fileptr);
				fileptr += 4;
				
				NumChars = BitConverter.ToInt32(FileBuffer, fileptr);
				fileptr += 4;
				
				Height = BitConverter.ToInt32(FileBuffer, fileptr);
				fileptr += 4;
				
				HeightOverBase = BitConverter.ToInt32(FileBuffer, fileptr);
				fileptr += 4;
				
				for(int i = 0; i < NumChars; i++)
				{
					short tmp_base = BitConverter.ToInt16(FileBuffer, fileptr);
					fileptr += 2;
						
					ushort tmp_ascii = BitConverter.ToUInt16(FileBuffer, fileptr);
					fileptr += 2;
					
					FntChar tmp = new FntChar(tmp_base, tmp_ascii);
					Chars.Add(tmp);
				}
				
				Texture.DataLength = BitConverter.ToInt32(FileBuffer, fileptr);
				fileptr += 4;
				
				Texture.Width = BitConverter.ToInt16(FileBuffer, fileptr);
				fileptr += 2;
				
				Texture.Height = BitConverter.ToInt16(FileBuffer, fileptr);
				fileptr += 2;
				
				Texture.Depth = BitConverter.ToInt16(FileBuffer, fileptr);
				fileptr += 2;
				
				Buffer.BlockCopy(FileBuffer, fileptr, Texture.Extra, 0, 6);
				fileptr += 6;
				
				PixelDataSize = (uint)(Texture.Width * Texture.Height * bitbytes(Texture.Depth));
				
				Pixels = new byte[PixelDataSize];
				Buffer.BlockCopy(FileBuffer, fileptr, Pixels, 0, (int)PixelDataSize);
				fileptr += (int)PixelDataSize;
				
				for(int i = 0; i < 256; i++)
				{
					CLUT[i].R = FileBuffer[fileptr];
					fileptr++;
					CLUT[i].G = FileBuffer[fileptr];
					fileptr++;
					CLUT[i].B = FileBuffer[fileptr];
					fileptr++;
					CLUT[i].A = FileBuffer[fileptr];
					fileptr++;
				}
				
				int num_subtex = BitConverter.ToInt32(FileBuffer, fileptr);
				fileptr += 4;
				
				for(int i = 0; i < num_subtex; i++)
				{
					Chars[i].PosX = BitConverter.ToUInt16(FileBuffer, fileptr);
					fileptr += 2;
					Chars[i].PosY = BitConverter.ToUInt16(FileBuffer, fileptr);
					fileptr += 2;
					Chars[i].Width = BitConverter.ToUInt16(FileBuffer, fileptr);
					fileptr += 2;
					Chars[i].Height = BitConverter.ToUInt16(FileBuffer, fileptr);
					fileptr += 2;
					
					Chars[i].BMP = new Bitmap(Chars[i].Width, Chars[i].Height, System.Drawing.Imaging.PixelFormat.Format32bppArgb);
					
					for(int i2 = Chars[i].PosY; i2 < (Chars[i].PosY - 1) + Chars[i].Height; i2++)
					{							
						for(int i3 = Chars[i].PosX; i3 < (Chars[i].PosX - 1) + Chars[i].Width; i3++)
						{
							int pt = (i2 * Texture.Width);
							pt += i3;
							
							Color c = Color.FromArgb(CLUT[Pixels[pt]].A, CLUT[Pixels[pt]].R, CLUT[Pixels[pt]].G, CLUT[Pixels[pt]].B);
							
							Chars[i].BMP.SetPixel(i3 - Chars[i].PosX, i2 - Chars[i].PosY, c);
						}
					}
				}
			}
			else
			{
				FileOpen = false;
			}
			
			return FileOpen;
		}
		
		public bool OpenTHUG2File(string a)
		{
			CloseFile();
			
			Filename = a;
			
			try
			{
				File = new FileStream(Filename, FileMode.Open);
			}
			catch
			{
				return false;
			}
			
			if(File.CanRead)
			{
				FileOpen = true;
				FileBuffer = new byte[File.Length];
				File.Read(FileBuffer, 0, FileBuffer.Length);
				
				int fileptr = 0;
				
				FileLength = BitConverter.ToInt32(FileBuffer, fileptr);
				fileptr += 8;
				
				NumChars = BitConverter.ToInt32(FileBuffer, fileptr);
				fileptr += 4;
				
				Height = BitConverter.ToInt32(FileBuffer, fileptr);
				fileptr += 4;
				
				HeightOverBase = BitConverter.ToInt32(FileBuffer, fileptr);
				fileptr += 4;
				
				for(int i = 0; i < NumChars; i++)
				{					
					short tmp_base = BitConverter.ToInt16(FileBuffer, fileptr);
					fileptr += 2;
						
					ushort tmp_ascii = BitConverter.ToUInt16(FileBuffer, fileptr);
					fileptr += 4;
					
					FntChar tmp = new FntChar(tmp_base, tmp_ascii);
					Chars.Add(tmp);
					
				}
				
				Texture.DataLength = BitConverter.ToInt32(FileBuffer, fileptr);
				fileptr += 4;
				
				Texture.Width = BitConverter.ToInt16(FileBuffer, fileptr);
				fileptr += 2;
				
				Texture.Height = BitConverter.ToInt16(FileBuffer, fileptr);
				fileptr += 2;
				
				Texture.Depth = BitConverter.ToInt16(FileBuffer, fileptr);
				fileptr += 2;
				
				Buffer.BlockCopy(FileBuffer, fileptr, Texture.Extra, 0, 6);
				fileptr += 6;
				
				PixelDataSize = (uint)(Texture.Width * Texture.Height * bitbytes(Texture.Depth));
				
				Pixels = new byte[PixelDataSize];
				Buffer.BlockCopy(FileBuffer, fileptr, Pixels, 0, (int)PixelDataSize);
				fileptr += (int)PixelDataSize;
				
				for(int i = 0; i < 256; i++)
				{
					CLUT[i].R = FileBuffer[fileptr];
					fileptr++;
					CLUT[i].G = FileBuffer[fileptr];
					fileptr++;
					CLUT[i].B = FileBuffer[fileptr];
					fileptr++;
					CLUT[i].A = FileBuffer[fileptr];
					fileptr++;
				}
				
				int num_subtex = BitConverter.ToInt32(FileBuffer, fileptr);
				fileptr += 4;
				
				for(int i = 0; i < num_subtex; i++)
				{
					Chars[i].PosX = BitConverter.ToUInt16(FileBuffer, fileptr);
					fileptr += 2;
					Chars[i].PosY = BitConverter.ToUInt16(FileBuffer, fileptr);
					fileptr += 2;
					Chars[i].Width = BitConverter.ToUInt16(FileBuffer, fileptr);
					fileptr += 2;
					Chars[i].Height = BitConverter.ToUInt16(FileBuffer, fileptr);
					fileptr += 2;
					
					Chars[i].BMP = new Bitmap(Chars[i].Width, Chars[i].Height, System.Drawing.Imaging.PixelFormat.Format32bppArgb);
					
					for(int i2 = Chars[i].PosY; i2 < (Chars[i].PosY - 1) + Chars[i].Height; i2++)
					{							
						for(int i3 = Chars[i].PosX; i3 < (Chars[i].PosX - 1) + Chars[i].Width; i3++)
						{
							int pt = (i2 * Texture.Width);
							pt += i3;
							
							Color c = Color.FromArgb(CLUT[Pixels[pt]].A, CLUT[Pixels[pt]].R, CLUT[Pixels[pt]].G, CLUT[Pixels[pt]].B);
							
							Chars[i].BMP.SetPixel(i3 - Chars[i].PosX, i2 - Chars[i].PosY, c);
						}
					}
				}
			}
			else
			{
				FileOpen = false;
			}
			
			return FileOpen;
		}
		
		public bool SaveTHUGFile(FileStream a)
		{
			bool success = false;
			int _fs = 0;
			byte[] _fb;
			byte[] _tb;
			int _fp = 0;
			int _ds = 0;
			
			if(FileOpen)
			{
				_fs = 1060 + (NumChars * 12) + (Texture.Width * Texture.Height);
				_fb = new byte[_fs];
				
				_tb = BitConverter.GetBytes(_fs);
				Buffer.BlockCopy(_tb, 0, _fb, _fp, 4);
				_fp += 4;
				
				_tb = BitConverter.GetBytes(NumChars);
				Buffer.BlockCopy(_tb, 0, _fb, _fp, 4);
				_fp += 4;
				
				_tb = BitConverter.GetBytes(Height);
				Buffer.BlockCopy(_tb, 0, _fb, _fp, 4);
				_fp += 4;
				
				_tb = BitConverter.GetBytes(HeightOverBase);
				Buffer.BlockCopy(_tb, 0, _fb, _fp, 4);
				_fp += 4;
				
				for(int i = 0; i < NumChars; i++)
				{
					_tb = BitConverter.GetBytes(Chars[i].Baseline);
					Buffer.BlockCopy(_tb, 0, _fb, _fp, 2);
					_fp += 2;
					
					_tb = BitConverter.GetBytes(Chars[i].Ascii);
					Buffer.BlockCopy(_tb, 0, _fb, _fp, 2);
					_fp += 2;
				}
				
				_ds = _fs - (16 + (NumChars * 4));
				
				_tb = BitConverter.GetBytes(_ds);
				Buffer.BlockCopy(_tb, 0, _fb, _fp, 4);
				_fp += 4;
				
				_tb = BitConverter.GetBytes(Texture.Width);
				Buffer.BlockCopy(_tb, 0, _fb, _fp, 2);
				_fp += 2;
				
				_tb = BitConverter.GetBytes(Texture.Height);
				Buffer.BlockCopy(_tb, 0, _fb, _fp, 2);
				_fp += 2;
				
				_tb = BitConverter.GetBytes(Texture.Depth);
				Buffer.BlockCopy(_tb, 0, _fb, _fp, 2);
				_fp += 2;
				
				Buffer.BlockCopy(Texture.Extra, 0, _fb, _fp, 6);
				_fp += 6;
				
				Buffer.BlockCopy(Pixels, 0, _fb, _fp, Pixels.Length);
				_fp += Pixels.Length;
				
				for(int i = 0; i < 256; i++)
				{
					_fb[_fp + (4 * i)] = CLUT[i].R;
					_fb[_fp + (4 * i) + 1] = CLUT[i].G;
					_fb[_fp + (4 * i) + 2] = CLUT[i].B;
					_fb[_fp + (4 * i) + 3] = CLUT[i].A;
				}
				
				_fp += 1024;
				
				_tb = BitConverter.GetBytes(NumChars);
				Buffer.BlockCopy(_tb, 0, _fb, _fp, 4);
				_fp += 4;
				
				for(int i = 0; i < NumChars; i++)
				{
					_tb = BitConverter.GetBytes(Chars[i].PosX);
					Buffer.BlockCopy(_tb, 0, _fb, _fp, 2);
					_fp += 2;
					
					_tb = BitConverter.GetBytes(Chars[i].PosY);
					Buffer.BlockCopy(_tb, 0, _fb, _fp, 2);
					_fp += 2;
					
					_tb = BitConverter.GetBytes(Chars[i].Width);
					Buffer.BlockCopy(_tb, 0, _fb, _fp, 2);
					_fp += 2;
					
					_tb = BitConverter.GetBytes(Chars[i].Height);
					Buffer.BlockCopy(_tb, 0, _fb, _fp, 2);
					_fp += 2;
				}
				
				try
				{
					a.Write(_fb, 0, _fs);
				}
				catch
				{
					success = false;
				}
			}
			else
			{
				success = false;
			}
			
			return success;
		}
		
		public bool SaveTHUG2File(FileStream a)
		{
			bool success = false;
			int _fs = 0;
			byte[] _fb;
			byte[] _tb;
			int _fp = 0;
			int _ds = 0;
			
			if(FileOpen)
			{
				_fs = 1064 + (NumChars * 14) + (Texture.Width * Texture.Height);
				_fb = new byte[_fs];
				
				_tb = BitConverter.GetBytes(_fs);
				Buffer.BlockCopy(_tb, 0, _fb, _fp, 4);
				_fp += 4;
				
				_tb = BitConverter.GetBytes((UInt32)1);
				Buffer.BlockCopy(_tb, 0, _fb, _fp, 4);
				_fp += 4;
				
				_tb = BitConverter.GetBytes(NumChars);
				Buffer.BlockCopy(_tb, 0, _fb, _fp, 4);
				_fp += 4;
				
				_tb = BitConverter.GetBytes(Height);
				Buffer.BlockCopy(_tb, 0, _fb, _fp, 4);
				_fp += 4;
				
				_tb = BitConverter.GetBytes(HeightOverBase);
				Buffer.BlockCopy(_tb, 0, _fb, _fp, 4);
				_fp += 4;
				
				for(int i = 0; i < NumChars; i++)
				{
					_tb = BitConverter.GetBytes(Chars[i].Baseline);
					Buffer.BlockCopy(_tb, 0, _fb, _fp, 2);
					_fp += 2;
					
					_tb = BitConverter.GetBytes(Chars[i].Ascii);
					Buffer.BlockCopy(_tb, 0, _fb, _fp, 2);
					_fp += 4;
				}
				
				_ds = _fs - (20 + (NumChars * 6));
				
				_tb = BitConverter.GetBytes(_ds);
				Buffer.BlockCopy(_tb, 0, _fb, _fp, 4);
				_fp += 4;
				
				_tb = BitConverter.GetBytes(Texture.Width);
				Buffer.BlockCopy(_tb, 0, _fb, _fp, 2);
				_fp += 2;
				
				_tb = BitConverter.GetBytes(Texture.Height);
				Buffer.BlockCopy(_tb, 0, _fb, _fp, 2);
				_fp += 2;
				
				_tb = BitConverter.GetBytes(Texture.Depth);
				Buffer.BlockCopy(_tb, 0, _fb, _fp, 2);
				_fp += 2;
				
				Buffer.BlockCopy(Texture.Extra, 0, _fb, _fp, 6);
				_fp += 6;
				
				Buffer.BlockCopy(Pixels, 0, _fb, _fp, Pixels.Length);
				_fp += Pixels.Length;
				
				for(int i = 0; i < 256; i++)
				{
					_fb[_fp + (4 * i)] = CLUT[i].R;
					_fb[_fp + (4 * i) + 1] = CLUT[i].G;
					_fb[_fp + (4 * i) + 2] = CLUT[i].B;
					_fb[_fp + (4 * i) + 3] = CLUT[i].A;
				}
				
				_fp += 1024;
				
				_tb = BitConverter.GetBytes(NumChars);
				Buffer.BlockCopy(_tb, 0, _fb, _fp, 4);
				_fp += 4;
				
				for(int i = 0; i < NumChars; i++)
				{
					_tb = BitConverter.GetBytes(Chars[i].PosX);
					Buffer.BlockCopy(_tb, 0, _fb, _fp, 2);
					_fp += 2;
					
					_tb = BitConverter.GetBytes(Chars[i].PosY);
					Buffer.BlockCopy(_tb, 0, _fb, _fp, 2);
					_fp += 2;
					
					_tb = BitConverter.GetBytes(Chars[i].Width);
					Buffer.BlockCopy(_tb, 0, _fb, _fp, 2);
					_fp += 2;
					
					_tb = BitConverter.GetBytes(Chars[i].Height);
					Buffer.BlockCopy(_tb, 0, _fb, _fp, 2);
					_fp += 2;
				}
				
				try
				{
					a.Write(_fb, 0, _fs);
				}
				catch
				{
					success = false;
				}
			}
			else
			{
				success = false;
			}
			
			return success;
		}
	}
}
